{*******************************************************
 Class:       TIDEMenuItem
 Created      9. 1998.
 Author:      Martin Waldenburg
 CopyRight:   Martin Waldenburg 1998 all rights reserved
 Description: MenuItem, which inserts and removes itself
              in the main menu of Delphi 4.
              Because of Delphi's loading behaviour it's
              not save to search for a sub menu item and
              then to insert the new item before or behind.
              Therefore you can set by name only one of the
              main menu items as the host and within it
              an index.
              If you don't set a name it will be inserted
              as a main menu item.

 Version:     1.0
 Status:      FreeWare
 Disclaimer:  This is provided as is, expressly without a
              warranty of any kind.
              You use it at your own risc.
 *******************************************************}

unit mIDEMenuItem;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, MWOTA;

type
  TIDEMenuItem = class(TMenuItem)
  private
    fHost: TMenuItem;
    fMain: Boolean;
  protected
    { Protected-Deklarationen}
  public
    constructor Create(HostName: string; AtIndex: Integer); reintroduce; overload;
    destructor Destroy; override;
  published
  end;

implementation

{ TIDEMenuItem }

constructor TIDEMenuItem.Create(HostName: string; AtIndex: Integer);
begin
  inherited Create(nil);
  if HostName <> '' then
  begin
    fMain:= False;
    fHost := TOTA.FindMenuItem(HostName);
    fHost.Insert(AtIndex, Self);
  end else
  begin
    fHost := nil;
    fMain:= True;
    TOTA.MenuInsert(AtIndex, Self);
  end;
end;

destructor TIDEMenuItem.Destroy;
var
  Index: Integer;
begin
  try
    if fHost <> nil then
    try
      Index := fHost.IndexOf(Self);
      if Index >= 0 then fHost.Delete(Index);
    finally
      fHost := nil;
    end else if fMain = True then TOTA.MenuRemove(Self);
  finally
  end;
  inherited Destroy;
end;

end.

