{+--------------------------------------------------------------------------+
 | Class:       TmProp
 | Created:     8.98
 | Author:      Martin Waldenburg
 | Copyright    1998 all rights reserved.
 | Description: Gives access to the published properties
 |              of a given class by name.
 |              Useful for workarounds for some of the broken
 |              parts of the new interfaces.
 | Version:     1.0
 | Status:      FreeWare
 | Disclaimer:
 | This is provided as is, expressly without a warranty of any kind.
 | You use it at your own risc.
 +--------------------------------------------------------------------------+}
unit mwProp;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  Dialogs,
  TypInfo;

type
  TmProp = class(Tobject)
  private
    Class procedure procTkArray(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkChar(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkClass(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkDynArray(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkEnumeration(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkFloat(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkInt64(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkInteger(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkInterface(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkMethod(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkLString(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkRecord(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkSet(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkString(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkUnknown(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkVariant(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkWChar(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
    Class procedure procTkWString(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
  protected
    { Protected-Deklarationen}
  public
    Class function GetPropByName(anObject: TObject; Name: String): PPropInfo;
    Class procedure GetPropInfoListByName(anObject: TObject; Name: String; List: TStringList);
    Class procedure GetPropNames(anObject: TObject; List: TStringList);
    Class procedure SetPropByName(anObject: TObject; Name: String; Const Value);
  published
    { Published-Deklarationen }
  end;

implementation

{ TmProp }

Class procedure TmProp.GetPropNames(anObject: TObject; List: TStringList);
var
  I, Count: Integer;
  PropList:PPropList;
  PKinds   : TTypeKinds;
begin
  List.Clear;
  PKinds:= [tkUnknown, tkInteger, tkChar, tkEnumeration, tkFloat,
    tkString, tkSet, tkClass, tkMethod, tkWChar, tkLString, tkWString,
    tkVariant, tkArray, tkRecord, tkInterface, tkInt64, tkDynArray];

  Count:= GetPropList(anObject.ClassInfo, PKinds, Nil);
  GetMem(PropList, Count *SizeOf(Pointer));
  GetPropList(anObject.ClassInfo, PKinds, PropList);
  for I:=0 to Count-1 do List.Add(PropList^[i].Name);
    FreeMem(PropList, Count *SizeOf(Pointer));
end;

Class function TmProp.GetPropByName(anObject: TObject;
  Name: String): PPropInfo;
var
  I, Count: Integer;
  PropList:PPropList;
  PKinds   : TTypeKinds;
begin
  Result:= nil;
  PKinds:= [tkUnknown, tkInteger, tkChar, tkEnumeration, tkFloat,
    tkString, tkSet, tkClass, tkMethod, tkWChar, tkLString, tkWString,
    tkVariant, tkArray, tkRecord, tkInterface, tkInt64, tkDynArray];

  Count:= GetPropList(anObject.ClassInfo, PKinds, Nil);
  GetMem(PropList, Count *SizeOf(Pointer));
  GetPropList(anObject.ClassInfo, PKinds, PropList);
  for I:=0 to Count-1 do
  begin
    if CompareText(PropList^[i].Name, Name) = 0 then
    begin
      Result:= PropList^[i];
      break;
    end;
  end;
    FreeMem(PropList, Count *SizeOf(Pointer));
end;

Class procedure TmProp.procTkArray(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
  end else
  begin
    List.Add('  property: ');
  end;
  List.Add('    TypeKind: Array');
end;

Class procedure TmProp.procTkChar(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  aLongInt: LongInt;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    List.Add('    TypeKind: Char');
    aLongInt:= GetOrdProp(anObject, PropInfo);
    List.Add('    Value: '+IntToStr(aLongInt));
  end else
  begin
    List.Add('  property: ');
    List.Add('    TypeKind: Char');
  end;
end;

Class procedure TmProp.procTkClass(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  TypeData: PTypeData;
begin
  if PropInfo <> nil then
  begin
    TypeData:= GetTypeData(PropInfo.PropType^);
    List.Add('  property: ' + PropInfo.Name);
    List.Add('    TypeKind: Class');
    List.Add('    ClassName: '+TypeData^.ClassType.ClassName + ' UnitName: '+TypeData^.UnitName);
    List.Add('    ClassParent: ' + TypeData^.ClassType.ClassParent.ClassName);
  end else
  begin
    List.Add('  property: ');
    List.Add('    TypeKind: Class');
  end;
  TypeData:= nil;
end;

Class procedure TmProp.procTkDynArray(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
  end else
  begin
    List.Add('  property: ');
  end;
  List.Add('    TypeKind: DynArray');
end;

Class procedure TmProp.procTkEnumeration(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  I, aLongInt: LongInt;
  TypeData: PTypeData;
  aString: String;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    aLongInt:= GetOrdProp(anObject, PropInfo);
    List.Add('    TypeKind: Enumeration');
    List.Add('    Value: '+ GetEnumName(PropInfo.PropType^, aLongInt));
    TypeData:= GetTypeData(PropInfo.PropType^);
    if (TypeData.BaseType<>nil) and (TypeData.BaseType^<>nil) and (TypeData.BaseType^^.Kind=tkEnumeration) then
    begin
      for i:=0 to GetTypeData(TypeData.BaseType^)^.MaxValue-GetTypeData(TypeData.BaseType^)^.MinValue do
      begin
        aString:= GetEnumName(TypeData.BaseType^,GetTypeData(TypeData.BaseType^)^.MinValue+i);
        List.Add('    Enums: '+IntToStr(i)+' '+ aString);
      end;
    end;
  end else
  begin
    List.Add('  property: ');
    List.Add('    TypeKind: Class');
  end;
  TypeData:= nil;
end;

Class procedure TmProp.procTkFloat(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  aExtended: Extended;
  TypeData: PTypeData;
begin
  aExtended:= 0;
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    aExtended:= GetFloatProp(anObject, PropInfo);
    TypeData:= GetTypeData(PropInfo.PropType^);
  end else
  begin
    List.Add('  property: ');
  end;
  List.Add('    TypeKind: Float');
  List.Add('    Value: '+ FloatToStr(aExtended));
  TypeData:= nil;
end;

Class procedure TmProp.procTkInt64(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  i64:Int64;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    List.Add('    TypeKind: Int64');
    i64:= GetInt64Prop(anObject, PropInfo);
    List.Add('    MinValue: '+IntToStr(i64));
  end else
  begin
    List.Add('  property: ');
    List.Add('    TypeKind: Int64');
  end;
end;

Class procedure TmProp.procTkInteger(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  aLongInt: LongInt;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    aLongInt:= GetOrdProp(anObject, PropInfo);
    List.Add('    TypeKind: Integer');
    List.Add('    Value: '+IntToStr(aLongInt));
  end else
  begin
    List.Add('  property: ');
    List.Add('    TypeKind: Integer');
  end;
end;

Class procedure TmProp.procTkInterface(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
  end else
  begin
    List.Add('  property: ');
  end;
  List.Add('    TypeKind: Interface');
end;

Class procedure TmProp.procTkLString(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  aString: String;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    aString:=GetStrProp(anObject, PropInfo);
  end else
  begin
    List.Add('  property: ');
    aString:=PString(anObject)^;
  end;
  List.Add('    TypeKind: String');
  List.Add('    Value: '+ aString);
end;

Class procedure TmProp.procTkMethod(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  aMethod: TMethod;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    aMethod:=GetMethodProp(anObject, PropInfo);
  end else
  begin
    List.Add('  property: ');
    List.Add('    TypeKind: Method');
  end;
  List.Add('    TypeKind: Method');
  if Assigned(aMethod.Code)then
  begin
    if Assigned(aMethod.Data) then
    List.Add('    CodeValue: '+ TObject(aMethod.Data).MethodName(aMethod.Code));
  end;
  if Assigned(aMethod.Data) then
  begin
    List.Add('    DataValue: ' + TObject(aMethod.Data).ClassName);
  end;
  if not(Assigned(aMethod.Code)) and not(Assigned(aMethod.Data)) then List.Add('    Value: nil');
end;

Class procedure TmProp.procTkRecord(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
  end else
  begin
    List.Add('  property: ');
  end;
  List.Add('  TypeKind: Record');
end;

Class procedure TmProp.procTkSet(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  I, aLongInt: LongInt;
  TypeData: PTypeData;
  aString: String;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    aLongInt:= GetOrdProp(anObject, PropInfo);
    TypeData:= GetTypeData(PropInfo.PropType^);
    List.Add('    TypeKind: Set');
    List.Add('    Value: '+IntToStr(aLongInt));
    if (TypeData.CompType<>nil) and (TypeData.CompType^<>nil) and (TypeData.CompType^^.Kind = tkEnumeration) then
    begin
      for I:= 0 to GetTypeData(TypeData.CompType^)^.MaxValue-GetTypeData(TypeData.CompType^)^.MinValue do
      begin
        aString:= GetEnumName(TypeData.CompType^, GetTypeData(TypeData.CompType^)^.MinValue+I);
        if ((1 shl i) and aLongInt)> 0 then
        begin
          List.Add('    In Set: ' + IntToStr(I)+' ' + aString);
        end else
        begin
        List.Add('    Not In Set: ' + IntToStr(I)+' ' + aString);
        end;
      end;
    end else
    begin
      for i:=0 to 31 do
      if ((1 shl i) and aLongInt)>0 then List.Add('    InSet: '+IntToStr(I));
    end;
  end else
  begin
    List.Add('  property: ');
    List.Add('    TypeKind: Set');
  end;
end;

Class procedure TmProp.procTkString(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  TypeData: PTypeData;
  aString: String;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    aString:= GetStrProp(anObject, PropInfo);
  end else
  begin
    List.Add('  property: ');
    aString:= PShortString(anObject)^;
  end;
  TypeData:= GetTypeData(PropInfo.PropType^);
  List.Add('    TypeKind: String');
  List.Add('    Value: '+aString);
  List.Add('    MaxSize: '+ IntToStr(TypeData.MaxLength));
end;

Class procedure TmProp.procTkUnknown(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
  end else
  begin
    List.Add('  property: ');
  end;
  List.Add('  TypeKind: Unknown');
end;

Class procedure TmProp.procTkVariant(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  aString: String;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    aString:= GetVariantProp(anObject, PropInfo);
  end else
  begin
    List.Add('  property: ');
    aString:= PString(anObject)^;
  end;
  List.Add('    TypeKind: Variant');
  List.Add('  Value: '+aString);
end;

Class procedure TmProp.procTkWChar(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  aLongInt: LongInt;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    aLongInt:= GetOrdProp(anObject, PropInfo);
    List.Add('    TypeKind: WChar');
    List.Add('    Value: '+IntToStr(aLongInt));
  end else
  begin
    List.Add('  property: ');
    List.Add('    TypeKind: WChar');
  end;
end;

Class procedure TmProp.procTkWString(anObject: TObject; PropInfo: PPropInfo; List: TStringList);
var
  aString: String;
begin
  if PropInfo <> nil then
  begin
    List.Add('  property: ' + PropInfo.Name);
    aString:=GetStrProp(anObject, PropInfo);
    List.Add('    TypeKind: WString');
    List.Add('    Value: '+ aString);
  end else
  begin
    List.Add('  property: ');
    List.Add('    TypeKind: WString');
  end;
end;

class procedure TmProp.SetPropByName(anObject: TObject;
  Name: String; Const Value);
var
  I, Count: Integer;
  PropList:PPropList;
  PKinds   : TTypeKinds;
begin
  PKinds:= [tkUnknown, tkInteger, tkChar, tkEnumeration, tkFloat,
    tkString, tkSet, tkClass, tkMethod, tkWChar, tkLString, tkWString,
    tkVariant, tkArray, tkRecord, tkInterface, tkInt64, tkDynArray];

  Count:= GetPropList(anObject.ClassInfo, PKinds, Nil);
  GetMem(PropList, Count *SizeOf(Pointer));
  GetPropList(anObject.ClassInfo, PKinds, PropList);
  for I:=0 to Count-1 do
  begin
    if CompareText(PropList^[i].Name, Name) = 0 then
    begin
      case PropList^[i].PropType^.Kind of
        tkArray: SetOrdProp(anObject, PropList^[i], LongInt(Value));
        tkChar: SetOrdProp(anObject, PropList^[i], LongInt(Value));
        tkClass: SetOrdProp(anObject, PropList^[i], LongInt(Value));
        tkDynArray: SetOrdProp(anObject, PropList^[i], LongInt(Value));
        tkEnumeration: SetOrdProp(anObject, PropList^[i], LongInt(Value));
        tkFloat: SetFloatProp(anObject, PropList^[i], Extended(Value));
        tkInt64: SetInt64Prop(anObject, PropList^[i], Int64(Value));
        tkInteger: SetOrdProp(anObject, PropList^[i], LongInt(Value));
        tkInterface: SetOrdProp(anObject, PropList^[i], LongInt(Value));
        tkMethod: SetMethodProp(anObject, PropList^[i], TMethod(Value));
        tkLString: SetStrProp(anObject, PropList^[i], String(Value));
        tkRecord: SetOrdProp(anObject, PropList^[i], LongInt(Value));
        tkSet: SetOrdProp(anObject, PropList^[i], LongInt(Value));
        tkString: SetStrProp(anObject, PropList^[i], String(Value));
        tkUnknown:;
        tkVariant: SetVariantProp(anObject, PropList^[i], Variant(Value));
        tkWChar: SetOrdProp(anObject, PropList^[i], LongInt(Value));
        tkWString: SetStrProp(anObject, PropList^[i], String(Value));
      end;
      break;
    end;
  end;
    FreeMem(PropList, Count *SizeOf(Pointer));
end;

class procedure TmProp.GetPropInfoListByName(anObject: TObject;
  Name: String; List: TStringList);
var
  PropInfo:PPropInfo;
begin
  PropInfo:= GetPropByName(anObject, Name);
    case PropInfo.PropType^.Kind of
      tkArray: procTkArray(anObject, PropInfo, List);
      tkChar: procTkChar(anObject, PropInfo, List);
      tkClass: procTkClass(anObject, PropInfo, List);
      tkDynArray: procTkDynArray(anObject, PropInfo, List);
      tkEnumeration: procTkEnumeration(anObject, PropInfo, List);
      tkFloat: procTkFloat(anObject, PropInfo, List);
      tkInt64: procTkInt64(anObject, PropInfo, List);
      tkInteger: procTkInteger(anObject, PropInfo, List);
      tkInterface: procTkInterface(anObject, PropInfo, List);
      tkMethod: procTkMethod(anObject, PropInfo, List);
      tkLString: procTkLString(anObject, PropInfo, List);
      tkRecord: procTkRecord(anObject, PropInfo, List);
      tkSet: procTkSet(anObject, PropInfo, List);
      tkString: procTkString(anObject, PropInfo, List);
      tkUnknown: procTkUnknown(anObject, PropInfo, List);
      tkVariant: procTkVariant(anObject, PropInfo, List);
      tkWChar: procTkWChar(anObject, PropInfo, List);
      tkWString: procTkWString(anObject, PropInfo, List);
    end;
end;

end.

